@@ -29,8 +29,17 @@
 #include <syslog.h>
 #include <security/pam_appl.h>
 #include <pwd.h>
+
 #ifdef __sun
 #include <fcntl.h>
+/*
+ * Solaris and other operating systems use const differently
+ * with PAM functions.  Using these defines avoids compiler
+ * warnings and needing to cast.
+ */
+#define GDM_PAM_QUAL
+#else
+#define GDM_PAM_QUAL const
 #endif
 
 #include <glib/gi18n.h>
@@ -40,6 +49,7 @@
 #include "slave.h"
 #include "verify.h"
 #include "errorgui.h"
+#include "getvt.h"
 
 #include "gdm-common.h"
 #include "gdm-log.h"
@@ -50,11 +60,20 @@
 #ifdef	HAVE_LOGINDEVPERM
 #include <libdevinfo.h>
 #endif	/* HAVE_LOGINDEVPERM */
+
 #ifdef  HAVE_ADT
 #include <bsm/adt.h>
 #include <bsm/adt_event.h>
 #endif	/* HAVE_ADT */
 
+#define  AU_FAILED 0
+#define  AU_SUCCESS 1
+#ifdef HAVE_LIBAUDIT
+#include <libaudit.h>
+#else
+#define log_to_audit_system(l,h,d,s)	do { ; } while (0)
+#endif
+
 /* Evil, but this way these things are passed to the child session */
 static pam_handle_t *pamh = NULL;
 
@@ -185,12 +204,16 @@ audit_fail_login (GdmDisplay *d, int pw_change, struct passwd *pwent,
 				"adt_start_session (ADT_login, ADT_FAILURE): %m");
 			return;
 		}
+
 		if (d->attached) {
+			gchar *device_name = gdm_slave_get_display_device (d);
+
 			/* login from the local host */
-			if (adt_load_ttyname ("/dev/console", &tid) != 0) {
+			if (adt_load_ttyname (device_name, &tid) != 0) {
 
 				syslog (LOG_AUTH | LOG_ALERT,
 					"adt_loadhostname (localhost): %m");
+			g_free (device_name);
 			}
 		} else {
 			/* login from a remote host */
@@ -200,6 +223,7 @@ audit_fail_login (GdmDisplay *d, int pw_change, struct passwd *pwent,
 					"adt_loadhostname (%s): %m", d->hostname);
 			}
 		}
+
 		if (adt_set_user (ah,
 				  pwent ? pwent->pw_uid : ADT_NO_ATTRIB,
 				  pwent ? pwent->pw_gid : ADT_NO_ATTRIB,
@@ -480,7 +504,8 @@ perhaps_translate_message (const char *msg)
  * authentication system and the actual greeter program */
 
 static int
-gdm_verify_pam_conv (int num_msg, struct pam_message **msg,
+gdm_verify_pam_conv (int num_msg,
+		     GDM_PAM_QUAL struct pam_message **msg,
 		     struct pam_response **resp,
 		     void *appdata_ptr)
 {
@@ -488,7 +513,7 @@ gdm_verify_pam_conv (int num_msg, struct pam_message **msg,
 	int i;
 	char *s = NULL;
 	struct pam_response *reply = NULL;
-	const void *p;
+	GDM_PAM_QUAL void *p;
 	const char *login;
 
 	if (pamh == NULL)
@@ -512,7 +537,7 @@ gdm_verify_pam_conv (int num_msg, struct pam_message **msg,
 	/* Here we set the login if it wasn't already set,
 	 * this is kind of anal, but this way we guarantee that
 	 * the greeter always is up to date on the login */
-	if (pam_get_item (pamh, PAM_USER, (void **)&p) == PAM_SUCCESS) {
+	if (pam_get_item (pamh, PAM_USER, &p) == PAM_SUCCESS) {
 		login = (const char *)p;
 		gdm_slave_greeter_ctl_no_ret (GDM_SETLOGIN, login);
 	}
@@ -530,7 +555,7 @@ gdm_verify_pam_conv (int num_msg, struct pam_message **msg,
 			/* PAM requested textual input with echo on */
 		case PAM_PROMPT_ECHO_ON:
 			if (strcmp (m, _("Username:")) == 0) {
-				if ( ve_string_empty (selected_user)) {
+				if (ve_string_empty (selected_user)) {
 					/* this is an evil hack, but really there is no way we'll
 					   know this is a username prompt.  However we SHOULD NOT
 					   rely on this working.  The pam modules can set their
@@ -583,6 +608,7 @@ gdm_verify_pam_conv (int num_msg, struct pam_message **msg,
 			reply[replies].resp_retcode = PAM_SUCCESS;
 			reply[replies].resp = NULL;
 			break;
+
 		case PAM_TEXT_INFO:
 			/* PAM sent a message that should displayed to the user */
 			gdm_slave_greeter_ctl_no_ret (GDM_MSG, m);
@@ -614,7 +640,8 @@ static struct pam_conv pamc = {
 static char *extra_standalone_message = NULL;
 
 static int
-gdm_verify_standalone_pam_conv (int num_msg, struct pam_message **msg,
+gdm_verify_standalone_pam_conv (int num_msg,
+				GDM_PAM_QUAL struct pam_message **msg,
 				struct pam_response **resp,
 				void *appdata_ptr)
 {
@@ -674,7 +701,7 @@ gdm_verify_standalone_pam_conv (int num_msg, struct pam_message **msg,
 				text = g_strdup (m);
 
 			/* PAM requested textual input with echo off */
-			question_msg = g_strdup_printf ("question_msg=%s$$echo=%d", text, TRUE);
+			question_msg = g_strdup_printf ("question_msg=%s$$echo=%d", text, FALSE);
 
 			gdm_slave_send_string (GDM_SOP_SHOW_QUESTION_DIALOG, question_msg);
 
@@ -738,6 +765,10 @@ create_pamh (GdmDisplay *d,
 	     const char *display,
 	     int *pamerr)
 {
+#ifdef __sun
+	gchar *device_name = NULL;
+#endif
+
 	if (display == NULL) {
 		gdm_error (_("Cannot setup pam handle with null display"));
 		return FALSE;
@@ -763,15 +794,21 @@ create_pamh (GdmDisplay *d,
 
 	/* Inform PAM of the user's tty */
 #ifdef __sun
-	if (d->attached)
-		(void) pam_set_item (pamh, PAM_TTY, "/dev/console");
-	else
+	device_name = gdm_slave_get_display_device (d);
+
+	if (device_name != NULL) {
+		(void) pam_set_item (pamh, PAM_TTY, device_name);
+		g_free (device_name);
+	} else {
 #endif	/* sun */
 		if ((*pamerr = pam_set_item (pamh, PAM_TTY, display)) != PAM_SUCCESS) {
 			if (gdm_slave_action_pending ())
 				gdm_error (_("Can't set PAM_TTY=%s"), display);
 			return FALSE;
 		}
+#ifdef __sun
+	}
+#endif
 
 	if ( ! d->attached) {
 		/* Only set RHOST if host is remote */
@@ -788,11 +825,57 @@ create_pamh (GdmDisplay *d,
 	return TRUE;
 }
 
+/**
+ * log_to_audit_system:
+ * @login: Name of user
+ * @hostname: Name of host machine
+ * @tty: Name of display 
+ * @success: 1 for success, 0 for failure
+ *
+ * Logs the success or failure of the login attempt with the linux kernel
+ * audit system. The intent is to capture failed events where the user
+ * fails authentication or otherwise is not permitted to login. There are
+ * many other places where pam could potentially fail and cause login to 
+ * fail, but these are system failures rather than the signs of an account
+ * being hacked.
+ *
+ * Returns nothing.
+ */
+
+#ifdef HAVE_LIBAUDIT
+static void 
+log_to_audit_system(const char *login,
+		    const char *hostname,
+		    const char *tty,
+		    gboolean success)
+{
+	struct passwd *pw;
+	char buf[64];
+	int audit_fd;
+
+	audit_fd = audit_open ();
+	if (login)
+		pw = getpwnam (login);
+	else {
+		login = "unknown";
+		pw = NULL;
+	}
+	if (pw) {
+		snprintf (buf, sizeof (buf), "uid=%d", pw->pw_uid);
+		audit_log_user_message (audit_fd, AUDIT_USER_LOGIN,
+				        buf, hostname, NULL, tty, (int)success);
+	} else {
+		snprintf (buf, sizeof (buf), "acct=%s", login);
+		audit_log_user_message (audit_fd, AUDIT_USER_LOGIN,
+				        buf, hostname, NULL, tty, (int)success);
+	}
+	close (audit_fd);
+}
+#endif
+
 /**
  * gdm_verify_user:
  * @username: Name of user or NULL if we should ask
- * @display: Name of display to register with the authentication system
- * @local: boolean if local
  * @allow_retry: boolean if we should allow retry logic to be enabled.
  *               We only want this to work for normal login, not for
  *               asking for the root password to cal the configurator.
@@ -806,15 +889,13 @@ create_pamh (GdmDisplay *d,
 gchar *
 gdm_verify_user (GdmDisplay *d,
 		 const char *username,
-		 const gchar *display,
-		 gboolean local,
 		 gboolean allow_retry)
 {
 	gint pamerr = 0;
 	struct passwd *pwent = NULL;
-	const void *p;
 	char *login, *passreq, *consoleonly;
 	char *pam_stack = NULL;
+	GDM_PAM_QUAL void *p;
 	int null_tok = 0;
 	gboolean credentials_set = FALSE;
 	gboolean error_msg_given = FALSE;
@@ -840,8 +921,8 @@ gdm_verify_user (GdmDisplay *d,
 	} else {
 		/* start the timer for timed logins */
 		if ( ! ve_string_empty (gdm_daemon_config_get_value_string (GDM_KEY_TIMED_LOGIN)) &&
-		     d->timed_login_ok &&
-		     (local || gdm_daemon_config_get_value_bool (GDM_KEY_ALLOW_REMOTE_AUTOLOGIN))) {
+		    d->timed_login_ok && (d->attached ||
+		    gdm_daemon_config_get_value_bool (GDM_KEY_ALLOW_REMOTE_AUTOLOGIN))) {
 			gdm_slave_greeter_ctl_no_ret (GDM_STARTTIMER, "");
 			started_timer = TRUE;
 		}
@@ -852,9 +933,9 @@ gdm_verify_user (GdmDisplay *d,
  authenticate_again:
 
 	if (prev_user && !login) {
-		login = g_strdup(prev_user);
+		login = g_strdup (prev_user);
 	} else if (login && !prev_user) {
-		prev_user = g_strdup(login);
+		prev_user = g_strdup (login);
 		auth_retries = 0;
 	} else if (login && prev_user && strcmp (login, prev_user)) {
 		g_free (prev_user);
@@ -868,9 +949,10 @@ gdm_verify_user (GdmDisplay *d,
 	 * PAM Stacks, in case one display should use a different
 	 * authentication mechanism than another display.
 	 */
-	pam_stack = gdm_daemon_config_get_value_string_per_display (GDM_KEY_PAM_STACK, (char *)display);
+	pam_stack = gdm_daemon_config_get_value_string_per_display (GDM_KEY_PAM_STACK,
+		(char *)d->name);
 
-	if ( ! create_pamh (d, pam_stack, login, &pamc, display, &pamerr)) {
+	if ( ! create_pamh (d, pam_stack, login, &pamc, d->name, &pamerr)) {
 		if (started_timer)
 			gdm_slave_greeter_ctl_no_ret (GDM_STOPTIMER, "");
 		g_free (pam_stack);
@@ -992,7 +1074,7 @@ gdm_verify_user (GdmDisplay *d,
 	g_free (prev_user);
 	prev_user = NULL;
 
-	if ((pamerr = pam_get_item (pamh, PAM_USER, (void **)&p)) != PAM_SUCCESS) {
+	if ((pamerr = pam_get_item (pamh, PAM_USER, &p)) != PAM_SUCCESS) {
 		login = NULL;
 		/* is not really an auth problem, but it will
 		   pretty much look as such, it shouldn't really
@@ -1026,12 +1108,11 @@ gdm_verify_user (GdmDisplay *d,
 		gdm_daemon_config_set_value_bool (GDM_KEY_ALLOW_REMOTE_ROOT, FALSE);
 
 	pwent = getpwnam (login);
-	if ( ( ! gdm_daemon_config_get_value_bool (GDM_KEY_ALLOW_ROOT) ||
-	       ( ! gdm_daemon_config_get_value_bool (GDM_KEY_ALLOW_REMOTE_ROOT) && ! local) ) &&
-	     pwent != NULL &&
-	     pwent->pw_uid == 0) {
+	if (( ! gdm_daemon_config_get_value_bool (GDM_KEY_ALLOW_ROOT) ||
+	    ( ! gdm_daemon_config_get_value_bool (GDM_KEY_ALLOW_REMOTE_ROOT) &&
+	      ! d->attached)) && pwent != NULL && pwent->pw_uid == 0) {
 		gdm_error (_("Root login disallowed on display '%s'"),
-			   display);
+			   d->name);
 		gdm_slave_greeter_ctl_no_ret (GDM_ERRBOX,
 					      _("\nThe system administrator "
 						"is not allowed to login "
@@ -1151,14 +1232,34 @@ gdm_verify_user (GdmDisplay *d,
 	cur_gdm_disp = NULL;
 
 #ifdef  HAVE_LOGINDEVPERM
-	if (d->attached)
-		(void) di_devperm_login ("/dev/console", pwent->pw_uid,
-					 pwent->pw_gid, NULL);
+	if (d->attached && d->type != TYPE_FLEXI_XNEST) {
+		gchar *device_name = gdm_slave_get_display_device (d);
+		/*
+		 * Only do logindevperm processing if /dev/console or
+		 * a device associated with a VT
+		 */
+		if (device_name != NULL &&
+		   (strncmp (device_name, "/dev/vt/", strlen ("/dev/vt/")) == 0 ||
+		    strcmp  (device_name, "/dev/console") == 0)) {
+			gdm_debug ("Logindevperm login for device %s", device_name);
+
+			(void) di_devperm_login (device_name, pwent->pw_uid,
+						 pwent->pw_gid, NULL);
+			g_free (device_name);
+		}
+	}
 #endif	/* HAVE_LOGINDEVPERM */
+
 #ifdef  HAVE_ADT
 	audit_success_login (pw_change, pwent);
 #endif  /* HAVE_ADT */
 
+	/*
+	 * Login succeeded.
+	 * This function is a no-op if libaudit is not present.
+	 */
+	log_to_audit_system(login, d->hostname, d->name, AU_SUCCESS);
+
 	return login;
 
  pamerr:
@@ -1166,10 +1267,20 @@ gdm_verify_user (GdmDisplay *d,
 	audit_fail_login (d, pw_change, pwent, pamerr);
 #endif	/* HAVE_ADT */
 
+	/*
+	 * Log the failed login attempt.
+	 * This function is a no-op if libaudit is not present.
+	 */
+	log_to_audit_system(login, d->hostname, d->name, AU_FAILED);
+
 	/* The verbose authentication is turned on, output the error
 	 * message from the PAM subsystem */
 	if ( ! error_msg_given &&
 	     gdm_slave_action_pending ()) {
+		gdm_slave_write_utmp_wtmp_record (d,
+					GDM_SESSION_RECORD_TYPE_FAILED_ATTEMPT,
+					login, getpid ());
+
 		/*
 		 * I'm not sure yet if I should display this message for any
 		 * other issues - heeten
@@ -1248,19 +1359,17 @@ gdm_verify_user (GdmDisplay *d,
 /**
  * gdm_verify_setup_user:
  * @login: The name of the user
- * @display: The name of the display
  *
  * This is used for auto loging in.  This just sets up the login
  * session for this user
  */
 
 gboolean
-gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
-		       char **new_login)
+gdm_verify_setup_user (GdmDisplay *d, const gchar *login, char **new_login)
 {
 	gint pamerr = 0;
 	struct passwd *pwent = NULL;
-	const void *p;
+	GDM_PAM_QUAL void *p;
 	char *passreq;
 	char *pam_stack = NULL;
 	char *pam_service_name = NULL;
@@ -1292,11 +1401,12 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	 * PAM Stacks, in case one display should use a different
 	 * authentication mechanism than another display.
 	 */
-	pam_stack = gdm_daemon_config_get_value_string_per_display (GDM_KEY_PAM_STACK, (char *)display);
+	pam_stack = gdm_daemon_config_get_value_string_per_display (GDM_KEY_PAM_STACK,
+		(char *)d->name);
 	pam_service_name = g_strdup_printf ("%s-autologin", pam_stack);
 
 	if ( ! create_pamh (d, pam_service_name, login, &standalone_pamc,
-			    display, &pamerr)) {
+			    d->name, &pamerr)) {
 		g_free (pam_stack);
 		g_free (pam_service_name);
 		goto setup_pamerr;
@@ -1324,7 +1434,7 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 		goto setup_pamerr;
 	}
 
-	if ((pamerr = pam_get_item (pamh, PAM_USER, (void **)&p)) != PAM_SUCCESS) {
+	if ((pamerr = pam_get_item (pamh, PAM_USER, &p)) != PAM_SUCCESS) {
 		/* is not really an auth problem, but it will
 		   pretty much look as such, it shouldn't really
 		   happen */
@@ -1334,7 +1444,7 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 					_("Authentication failed"));
 		goto setup_pamerr;
 	}
-	after_login = (const char *)p;
+	after_login = p;
 
 	if (after_login != NULL /* should never be */ &&
 	    strcmp (after_login, login) != 0) {
@@ -1451,22 +1561,46 @@ gdm_verify_setup_user (GdmDisplay *d, const gchar *login, const gchar *display,
 	extra_standalone_message = NULL;
 
 #ifdef  HAVE_LOGINDEVPERM
-	if (d->attached)
-		(void) di_devperm_login ("/dev/console", pwent->pw_uid,
-					 pwent->pw_gid, NULL);
+	if (d->attached && d->type != TYPE_FLEXI_XNEST) {
+		gchar *device_name = gdm_slave_get_display_device (d);
+		/*
+		 * Only do logindevperm processing if /dev/console or
+		 * a device associated with a VT
+		 */
+		if (device_name != NULL &&
+		   (strncmp (device_name, "/dev/vt/", strlen ("/dev/vt/")) == 0 ||
+		    strcmp  (device_name, "/dev/console") == 0)) {
+			gdm_debug ("Logindevperm login for device %s", device_name);
+			(void) di_devperm_login (device_name, pwent->pw_uid,
+						 pwent->pw_gid, NULL);
+			g_free (device_name);
+		}
+	}
 #endif	/* HAVE_LOGINDEVPERM */
+
 #ifdef  HAVE_ADT
 	audit_success_login (pw_change, pwent);
 #endif	/* HAVE_ADT */
 
+	/*
+	 * Login succeeded.
+	 * This function is a no-op if libaudit is not present
+	 */
+	log_to_audit_system(login, d->hostname, d->name, AU_SUCCESS);
+
 	return TRUE;
 
  setup_pamerr:
-
 #ifdef  HAVE_ADT
 	audit_fail_login (d, pw_change, pwent, pamerr);
 #endif	/* HAVE_ADT */
 
+	/*
+	 * Log the failed login attempt.
+	 * This function is a no-op if libaudit is not present
+	 */
+	log_to_audit_system(login, d->hostname, d->name, AU_FAILED);
+
 	did_setcred = FALSE;
 	opened_session = FALSE;
 	if (pamh != NULL) {
@@ -1555,12 +1689,20 @@ gdm_verify_cleanup (GdmDisplay *d)
 		pam_end (tmp_pamh, pamerr);
 
 #ifdef  HAVE_LOGINDEVPERM
-		if (old_opened_session && old_did_setcred && d->attached) {
-			(void) di_devperm_logout ("/dev/console");
-			/* give it back to gdm user */
-			(void) di_devperm_login ("/dev/console",
-						 gdm_daemon_config_get_gdmuid (),
-						 gdm_daemon_config_get_gdmgid (), NULL);
+		if (old_opened_session && old_did_setcred &&
+		    d->attached && d->type != TYPE_FLEXI_XNEST) {
+			gchar *device_name = gdm_slave_get_display_device (d);
+			/*
+			 * Only do logindevperm processing if /dev/console or
+			 * a device associated with a VT
+			 */
+			if (device_name != NULL &&
+			   (strncmp (device_name, "/dev/vt/", strlen ("/dev/vt/")) == 0 ||
+			    strcmp  (device_name, "/dev/console") == 0)) {
+				gdm_debug ("Logindevperm logout for device %s", device_name);
+				(void) di_devperm_logout (device_name);
+				g_free (device_name);
+			}
 		}
 #endif  /* HAVE_LOGINDEVPERM */
 
@@ -1580,41 +1722,6 @@ gdm_verify_cleanup (GdmDisplay *d)
 	gdm_reset_limits ();
 }
 
-/**
- * gdm_verify_check:
- *
- * Check that the authentication system is correctly configured.
- * Not very smart, perhaps we should just whack this.
- *
- * Aborts daemon on error
- */
-
-void
-gdm_verify_check (void)
-{
-	pam_handle_t *ph = NULL;
-
-	if (pam_start (gdm_daemon_config_get_value_string (GDM_KEY_PAM_STACK), NULL,
-		       &standalone_pamc, &ph) != PAM_SUCCESS) {
-		ph = NULL; /* be anal */
-
-                gdm_log_shutdown ();
-                gdm_log_init ();
-
-		if (gdm_daemon_config_get_value_bool (GDM_KEY_CONSOLE_NOTIFY))
-			gdm_text_message_dialog
-				(C_(N_("Can't find PAM configuration for GDM.")));
-		gdm_fail ("gdm_verify_check: %s",
-			  _("Can't find PAM configuration for GDM."));
-	}
-
-	if (ph != NULL)
-		pam_end (ph, PAM_SUCCESS);
-
-        gdm_log_shutdown ();
-        gdm_log_init ();
-}
-
 /* used in pam */
 gboolean
 gdm_verify_setup_env (GdmDisplay *d)
