@@ -159,14 +159,20 @@ make_menubar (void)
 	menu = gtk_menu_new ();
 	gtk_menu_item_set_submenu (GTK_MENU_ITEM (w), menu);
 
-	w = gtk_menu_item_new_with_mnemonic (_("Select _Language..."));
+	w = gtk_image_menu_item_new_with_mnemonic (_("Select _Language..."));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w), 
+		gtk_image_new_from_icon_name ("preferences-desktop-locale", GTK_ICON_SIZE_MENU));
+
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 	gtk_widget_show (GTK_WIDGET (w));
 	g_signal_connect (G_OBJECT (w), "activate",
 			  G_CALLBACK (activate_button),
 			  "language_button");
 
-	w = gtk_menu_item_new_with_mnemonic (_("Select _Session..."));
+	w = gtk_image_menu_item_new_with_mnemonic (_("Select _Session..."));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w), 
+		gtk_image_new_from_icon_name ("user-desktop", GTK_ICON_SIZE_MENU));
+
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), w);
 	gtk_widget_show (GTK_WIDGET (w));
 	g_signal_connect (G_OBJECT (w), "activate",
@@ -180,9 +186,13 @@ make_menubar (void)
 	 * flexi, even if not local (non-local xnest).  and Disconnect
 	 * only for xdmcp */
 	if ( ! ve_string_empty (g_getenv ("GDM_FLEXI_SERVER"))) {
-		w = gtk_menu_item_new_with_mnemonic (_("_Quit"));
+		w = gtk_image_menu_item_new_with_mnemonic (_("_Quit"));
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w), 
+			gtk_image_new_from_icon_name ("system-log-out", GTK_ICON_SIZE_MENU));
 	} else if (ve_string_empty (g_getenv ("GDM_IS_LOCAL"))) {
-		w = gtk_menu_item_new_with_mnemonic (_("D_isconnect"));
+		w = gtk_image_menu_item_new_with_mnemonic (_("D_isconnect"));
+		gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (w), 
+			gtk_image_new_from_icon_name ("system-log-out", GTK_ICON_SIZE_MENU));
 	} else {
 		w = NULL;
 	}
