@@ -183,6 +183,17 @@ pam_key_release_event (GtkWidget *entry, GdkEventKey *event, gpointer data)
        const char *login_string;
        GtkWidget *entry = GNOME_CANVAS_WIDGET (entry_info->item)->widget;
 
+       /*
+        * Allow tab to trigger enter; but not <modifier>-tab
+        */
+       if ((event->keyval == GDK_Tab ||
+            event->keyval == GDK_KP_Tab) &&
+           (event->state & (GDK_CONTROL_MASK|GDK_MOD1_MASK|GDK_SHIFT_MASK)) == 0)
+          {
+		greeter_item_pam_login (GTK_ENTRY (entry), entry_info);
+		return TRUE;
+           }
+
        if (gtk_ok_button != NULL)
           {
              /*
